@@ -443,6 +443,9 @@ sim_pow <- function(fixed, random = NULL, random3 = NULL, fixed_param,
 #'  as the level two or three sample size. These are specified as a named list in which
 #'  level two sample size is controlled via "level2" and level three sample size is 
 #'  controlled via "level3".
+#' @param outcome_type A vector specifying the type of outcome, must be either
+#'   logistic or poisson. Logitstic outcome will be 0/1 and poisson outcome will
+#'   be counts.
 #' @param missing TRUE/FALSE flag indicating whether missing data should be 
 #'  simulated.
 #' @param missing_args Additional missing arguments to pass to the missing_data 
@@ -491,6 +494,7 @@ sim_pow <- function(fixed, random = NULL, random3 = NULL, fixed_param,
 #' power_out <- sim_pow_glm(fixed = fixed, fixed_param = fixed_param, 
 #'                          cov_param = cov_param, 
 #'                          n = n, data_str = "single", 
+#'                          outcome_type = 'logistic', 
 #'                          pow_param = pow_param, alpha = alpha,
 #'                          pow_dist = pow_dist, pow_tail = pow_tail, 
 #'                          replicates = replicates, raw_power = FALSE)
@@ -502,6 +506,7 @@ sim_pow_glm <- function(fixed, random = NULL, random3 = NULL, fixed_param,
                     data_str, cor_vars = NULL, fact_vars = list(NULL), 
                     unbal = list("level2" = FALSE, "level3" = FALSE), 
                     unbal_design = list("level2" = NULL, "level3" = NULL),
+                    outcome_type, 
                     missing = FALSE, missing_args = list(NULL),
                   pow_param, alpha, pow_dist = c("z", "t"), pow_tail = c(1, 2), 
                     replicates, terms_vary = NULL, raw_power = TRUE, 
@@ -514,6 +519,7 @@ sim_pow_glm <- function(fixed, random = NULL, random3 = NULL, fixed_param,
                cov_param = cov_param, k = k, n = n, p = p, 
                data_str = data_str, cor_vars = cor_vars, 
                fact_vars = fact_vars, unbal = unbal, unbal_design = unbal_design,
+               outcome_type = outcome_type, 
                missing = missing, missing_args = missing_args,
                pow_param = pow_param, alpha = alpha, pow_dist = pow_dist, 
                pow_tail = pow_tail, glm_fit_mod = glm_fit_mod, 
